import os
import re
import unittest
from types import ModuleType
from typing import List

from pyshex import ShExEvaluator
from pyshex.shex_evaluator import EvaluationResult
from rdflib import Graph, Namespace, URIRef

from biolinkml import METAMODEL_NAMESPACE, LOCAL_SHEXJ_FILE_NAME, LOCAL_METAMODEL_LDCONTEXT_FILE, MODULE_DIR
from biolinkml.generators.csvgen import CsvGenerator
from biolinkml.generators.dotgen import DotGenerator
from biolinkml.generators.golrgen import GolrSchemaGenerator
from biolinkml.generators.graphqlgen import GraphqlGenerator
from biolinkml.generators.jsonldcontextgen import ContextGenerator
from biolinkml.generators.jsonldgen import JSONLDGenerator
from biolinkml.generators.jsonschemagen import JsonSchemaGenerator
from biolinkml.generators.markdowngen import MarkdownGenerator
from biolinkml.generators.namespacegen import NamespaceGenerator
from biolinkml.generators.owlgen import OwlSchemaGenerator
from biolinkml.generators.protogen import ProtoGenerator
from biolinkml.generators.pythongen import PythonGenerator
from biolinkml.generators.rdfgen import RDFGenerator
from biolinkml.generators.shexgen import ShExGenerator
from biolinkml.utils.formatutils import shex_results_as_string
from biolinkml.utils.context_utils import parse_import_map
from tests.utils.metadata_filters import metadata_filter
from tests.utils.generator_utils import GeneratorTestCase

BIOLINK_NS = Namespace("https://w3id.org/biolink/vocab/")

# ShEx validation of the biolink model takes a loooong time, so we only do it on rare occasions
DO_SHEX_VALIDATION = False


class CurrentBiolinkModelTestCase(GeneratorTestCase):
    cwd = os.path.abspath(os.path.dirname(__file__))

    source_path = os.path.join(cwd, 'source')
    target_path = os.path.join(cwd, 'target')
    model_path = os.path.join(cwd, 'yaml')
    model_name = 'biolink-model'
    importmap = parse_import_map(os.path.join(model_path, 'biolink-model-map.json'), MODULE_DIR)

    def tearDown(self) -> None:
        self.output_name = None

    def test_biolink_python(self):
        """ Test the python generator for the biolink model """
        self.output_name = 'model'
        self.single_file_generator('py', PythonGenerator, generator_args={'emit_metadata': True}, filtr=metadata_filter)

        # Make sure the python is valid
        with open(os.path.join(self.source_path, f'{self.output_name}.py')) as f:
            pydata = f.read()
        spec = compile(pydata, 'test', 'exec')
        module = ModuleType('test')
        exec(spec, module.__dict__)

    def test_biolink_markdown(self):
        """ Test the markdown generator for the biolink model """
        # NOTE: One may ocassionaly want to set image_dir to True and look at the target/markdown/images directory
        #       to make sure that images are still begin generated correctly
        self.directory_generator('markdown', MarkdownGenerator, serialize_args=dict(image_dir=False))

    def test_biolink_tsv(self):
        """ Test the tsv generator for the biolink model """
        def filtr(s: str) -> str:
            return s.replace('\r\n', '\n')
        self.single_file_generator('tsv', CsvGenerator, format="tsv", filtr=filtr)

    @unittest.skipIf(True, "Skip graphviz since it depends on dot and breaks on the automated GitHub Actions tests")
    def test_biolink_graphviz(self):
        """ Test the dotty generator for the biolink model """
        # We don't do the comparison step because different graphviz libraries generate slightly different binary output
        # We also don't commit the results -- the graphviz output is in .gitignore
        self.directory_generator('graphviz', DotGenerator, skip_compare_step=True)

    def test_biolink_golr(self):
        """ Test the golr schema generator for the biolink model """
        self.directory_generator('golr', GolrSchemaGenerator)

    def test_biolink_graphql(self):
        """ Test the graphql schema generator for the biolink model """
        self.single_file_generator('graphql', GraphqlGenerator)

    def test_biolink_jsonld(self):
        """ Test the jsonld schema generator for the biolink model """
        def filtr(s: str) -> str:
            return re.sub(r'"source_file_date": ".*?",', '"source_file_date": "2019-01-01-12:00",',
                          re.sub(r'"generation_date": ".*?",', '"generation_date": "2019-01-01 12:00",', s))
        self.single_file_generator('jsonld', JSONLDGenerator, filtr=filtr)

    def test_biolink_context(self):
        """ Test the jsonld context generator for the biolink model """
        def filtr(s: str) -> str:
            return re.sub(r'Generation date: .*?\\n', r'Generation date: \\n',
                          re.sub(r' version: .*?\\n', r' version: \\n', s))
        self.single_file_generator('context.jsonld', ContextGenerator, filtr=filtr)
        # Generate a second copy with native identifiers
        self.single_file_generator('context.native.jsonld', ContextGenerator, filtr=filtr,
                                   generator_args=dict(useuris=False))

    def test_biolink_json_schema(self):
        """ Test the jsonld context generator for the biolink model """
        self.single_file_generator('json.schema', JsonSchemaGenerator)

    def test_biolink_owl_schema(self):
        """ Test the owl schema generator for the biolink model """
        self.single_file_generator('owl', OwlSchemaGenerator, comparator=GeneratorTestCase.rdf_comparator)
        # Generate a second copy with native identifiers
        self.single_file_generator('native.owl', OwlSchemaGenerator, comparator=GeneratorTestCase.rdf_comparator,
                                   generator_args=dict(useuris=False))

    def test_biolink_proto(self):
        """ Test the proto schema generator for the biolink model """
        self.single_file_generator('proto', ProtoGenerator)

    def test_biolink_namespaces(self):
        """ Test the python generator for the biolink model """
        self.output_name = 'namespaces'
        self.single_file_generator('py', NamespaceGenerator, generator_args={'emit_metadata': True}, filtr=metadata_filter)

        # Make sure the python is valid
        with open(os.path.join(self.source_path, f'{self.output_name}.py')) as f:
            pydata = f.read()
        spec = compile(pydata, 'test', 'exec')
        module = ModuleType('test')
        exec(spec, module.__dict__)

    @staticmethod
    def _evaluate_shex_results(results: List[EvaluationResult], printit: bool=True) -> bool:
        """
        Check the results of the ShEx evaluation
        :param results: evaluate output
        :return: success indicator
        """
        success = all(r.result for r in results)
        if not success and printit:
            for r in results:
                if not r.result:
                    print(r.reason)
        return success

    @unittest.skipIf(True, "Skip the test temporarily. The Turtle file is still not matching. ")
    def test_biolink_rdf(self):
        """ Test the rdf generator for the biolink model """
        self.single_file_generator('ttl', RDFGenerator,
                                   serialize_args=dict(context=["https://w3id.org/biolink/biolink-model/context.jsonld"]),
                                   comparator=GeneratorTestCase.rdf_comparator)

        # Validate the RDF against the Biolink ShEx
        if DO_SHEX_VALIDATION:
            g = Graph()
            rdf_file = os.path.join(self.source_path, 'biolink_model.ttl')
            g.load(rdf_file, format='ttl')
            focus = BIOLINK_NS.biolink_model
            start = METAMODEL_NAMESPACE.SchemaDefinition
            results = ShExEvaluator(g, LOCAL_SHEXJ_FILE_NAME, focus, start).evaluate(debug=False)
            self.assertTrue(self._evaluate_shex_results(results))

        else:
            print("*** RDF Model validation step was skipped. See: test_biolink_model.DO_SHEX_VALIDATION to run it")

    def test_biolink_shex(self):
        """ Just Generate the ShEx file untested """
        self.single_file_generator('shex', ShExGenerator)
        self.single_file_generator('shexj', ShExGenerator, format='json')
        # Generate native ShEx
        self.single_file_generator('native.shex', ShExGenerator, generator_args=dict(useuris=False))
        self.single_file_generator('native.shexj', ShExGenerator, format='json',
                                   generator_args=dict(useuris=False))

    def test_biolink_shex_incorrect_rdf(self):
        """ Test some non-conforming RDF  """
        self.single_file_generator('shexj', ShExGenerator, format='json')
        shex_file = os.path.join(self.source_path, 'biolink-model.shexj')
        data_dir = os.path.join(self.cwd, 'data')

        focus = "http://identifiers.org/drugbank:DB00005"
        start = BIOLINK_NS.Drug
        evaluator = ShExEvaluator(None, shex_file, focus, start)

        # incorrect.ttl has 16 error lines (more or less).
        rdf_file = os.path.join(data_dir, 'incorrect.ttl')
        errs_file = os.path.join(data_dir, 'incorrect.errs')
        results = evaluator.evaluate(rdf_file)
        self.assertFalse(self._evaluate_shex_results(results, printit=False))
        self.assertEqual(1, len(results))
        self.assertTrue('Unmatched triples in CLOSED shape' in results[0].reason)
        ntabs = results[0].reason.count('\n\t')
        self.assertEqual(13, ntabs)
        if not os.path.exists(errs_file):
            with open(errs_file, 'w') as f:
                f.write(shex_results_as_string(results[0]))
        # TODO:
        #     self.assertTrue(False, f"{errs_file} created - run test again")
        # else:
        #     with open(errs_file) as f:
        #         expected = f.read()
        #     self.assertEqual(expected, shex_results_as_string(results[0]))

    @unittest.skipIf(not DO_SHEX_VALIDATION, "Skipping ShEx Validation")
    def test_biolink_correct_rdf(self):
        """ Test some conforming RDF  """
        self.single_file_generator('shexj', ShExGenerator, format='json')    # Make sure ShEx is current

        shex_file = os.path.join(self.source_path, 'biolink-model.shexj')
        data_dir = os.path.join(self.cwd, 'data')

        focus = "http://identifiers.org/drugbank:DB00005"
        start = BIOLINK_NS.Drug
        evaluator = ShExEvaluator(None, shex_file, focus, start)

        rdf_file = os.path.join(data_dir, 'probe.ttl')
        results = evaluator.evaluate(rdf_file, debug=False)
        self.assertTrue(self._evaluate_shex_results(results))


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from biolinkml.generators.csvgen import CsvGenerator
from tests.test_issues import sourcedir


class Issue38UnitTest(unittest.TestCase):
    def test_domain_slots(self):
        """ Subsets need to be imported as well """
        CsvGenerator(os.path.join(sourcedir, 'issue_38.yaml')).serialize()
        # We never get here if the imports fails
        self.assertEqual(True, True)


if __name__ == '__main__':
    unittest.main()



"""
Generate dotfiles
"""
import os
from typing import TextIO, Union, List, Optional

import click
from graphviz import Digraph, FORMATS

from biolinkml.meta import SchemaDefinition, ClassDefinition, SlotDefinition
from biolinkml.utils.formatutils import underscore
from biolinkml.utils.generator import Generator, shared_arguments

valid_formats = sorted(list(FORMATS))


class DotGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.1.1"
    valid_formats: List[str] = ['png'] + valid_formats
    visit_all_class_slots = True

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], **kwargs) -> None:
        super().__init__(schema, **kwargs)
        self.classnames: Optional[List[str]] = None
        self.filename: Optional[str] = None
        self.dirname: Optional[str] = None
        self.filedot: Optional[Digraph] = None
        self.classdot: Optional[Digraph] = None
        self.cls_subj: Optional[SlotDefinition] = None
        self.cls_obj: Optional[SlotDefinition] = None

    def visit_schema(self, classname: Optional[List[str]] = None, directory: Optional[str] = None,
                     filename: Optional[str] = None, **_) -> None:
        self.classnames = [] if classname is None else list(classname)
        for classname in self.classnames:
            if classname not in self.schema.classes:
                raise ValueError(f"Unknown class name: {classname}")
        self.filename = filename
        self.dirname = directory
        if filename:
            self.filedot = Digraph(comment=self.schema.name)
        if directory:
            os.makedirs(directory, exist_ok=True)

    def end_schema(self, **_) -> None:
        if self.filedot:
            self.filedot.render(self.filename, self.dirname, view=False, cleanup=True, format=self.format)

    def visit_class(self, cls: ClassDefinition) -> bool:
        if self.classnames and cls.name not in self.classnames:
            return False
        if self.dirname:
            self.classdot = Digraph(comment=self.schema.name)
        self.node(cls.name, cls.name)
        if cls.is_a:
            self.edge(cls.name, cls.is_a, label="is_a")
        for mixin in cls.mixins:
            self.edge(cls.name, mixin, label="uses")
        self.cls_subj = self.cls_obj = None
        return True

    def end_class(self, cls: ClassDefinition) -> None:
        if self.cls_subj and self.cls_obj:
            rnode = 'relation'
            self.edge(self.aliased_slot_name(self.cls_subj), self.aliased_slot_name(self.cls_obj), label=rnode)
            self.edge(self.aliased_slot_name(self.cls_subj), rnode, style='dotted')
            self.edge(self.aliased_slot_name(self.cls_obj), rnode, style='dotted')
        if self.classdot:
            self.classdot.render(underscore(cls.name), self.dirname, view=False, cleanup=True, format=self.format)

    def visit_class_slot(self, cls: ClassDefinition, aliased_slot_name: str, slot: SlotDefinition):
        if aliased_slot_name == 'subject':
            self.cls_subj = slot
        elif aliased_slot_name == 'object':
            self.cls_obj = slot
        color = 'blue' if slot.name in cls.slots else 'black'
        style = 'dashed' if slot.alias in cls.slots else 'solid'
        self.edge(cls.name, aliased_slot_name, label=aliased_slot_name, color=color, style=style)
        srange = (slot.range if slot.range else 'Thing')
        self.node(slot.name, srange, color=color)

    def node(self, *args, **kwargs) -> None:
        if self.classdot:
            self.classdot.node(*args, **kwargs)
        if self.filedot:
            self.filedot.node(*args, **kwargs)

    def edge(self, *args, **kwargs) -> None:
        if self.classdot:
            self.classdot.edge(*args, **kwargs)
        if self.filedot:
            self.filedot.edge(*args, **kwargs)


@shared_arguments(DotGenerator)
@click.command()
@click.option("--directory", "-d", help="Output directory - if supplied, a graph per class will be generated")
@click.option("--out", "-o", help="Target file -- if supplied, one large graph will be generated")
@click.option("--classname", "-c", default=None, multiple=True, help="Class(es) to transform")
def cli(yamlfile, out, **args):
    """ Generate graphviz representations of the biolink model """
    DotGenerator(yamlfile, **args).serialize(filename=out, **args)



from dataclasses import MISSING, _HAS_DEFAULT_FACTORY, _POST_INIT_NAME, _FIELD_INITVAR, _init_param, _field_init, _create_fn


def dataclasses_init_fn_with_kwargs(fields, frozen, has_post_init, self_name):
    # fields contains both real fields and InitVar pseudo-fields.

    # Make sure we don't have fields without defaults following fields
    # with defaults.  This actually would be caught when exec-ing the
    # function source code, but catching it here gives a better error
    # message, and future-proofs us in case we build up the function
    # using ast.
    seen_default = False
    for f in fields:
        # Only consider fields in the __init__ call.
        if f.init:
            if not (f.default is MISSING and f.default_factory is MISSING):
                seen_default = True
            elif seen_default:
                raise TypeError(f'non-default argument {f.name!r} '
                                'follows default argument')

    globals = {'MISSING': MISSING,
               '_HAS_DEFAULT_FACTORY': _HAS_DEFAULT_FACTORY}

    body_lines = []
    for f in fields:
        line = _field_init(f, frozen, globals, self_name)
        # line is None means that this field doesn't require
        # initialization (it's a pseudo-field).  Just skip it.
        if line:
            body_lines.append(line)

    # Does this class have a post-init function?
    if has_post_init:
        params_str = ','.join(f.name for f in fields
                              if f._field_type is _FIELD_INITVAR)
        body_lines.append(f'{self_name}.{_POST_INIT_NAME}({params_str}{", " if params_str else ""} **kwargs)')

    # If no body lines, use 'pass'.
    if not body_lines:
        body_lines = ['pass']

    locals = {f'_type_{f.name}': f.type for f in fields}
    return _create_fn('__init__',
                      [self_name] + [_init_param(f) for f in fields if f.init] + ["**kwargs"],
                      body_lines,
                      locals=locals,
                      globals=globals,
                      return_type=None)



import re
import unittest

# This has to occur post ClickTestCase
from functools import reduce
from typing import List, Tuple

import click

from biolinkml.generators.owlgen import cli
from tests import source_yaml_path
from tests.test_scripts.clicktestcase import ClickTestCase

repl: List[Tuple[str, str]] = [
    (r'\s*meta:generation_date ".*" ;', 'meta:generation_date "Fri Jan 25 14:22:29 2019" ;'),
    (r'\s*meta:source_file_date ".*" ;', 'meta:source_file_date "Fri Jan 25 14:22:29 2019" ;')
]


def filtr(txt: str) -> str:
    return reduce(lambda s, expr: re.sub(expr[0], expr[1], s, flags=re.MULTILINE), repl, txt)


class GenOWLTestCase(ClickTestCase):
    testdir = "genowl"
    click_ep = cli
    prog_name = "gen-owl"

    def test_help(self):
        self.do_test("--help", 'help', bypass_soft_compare=True)

    def test_meta(self):
        self.maxDiff = None
        self.do_test(source_yaml_path, 'meta_owl.ttl', filtr=filtr, comparator=ClickTestCase.rdf_comparator)
        self.soft_compare = "Files differ but json-ld OWL representations have bnodes and ordering issues"
        self.do_test(source_yaml_path + ' -f json-ld', 'meta_owl.jsonld')
        self.soft_compare = None
        self.do_test(source_yaml_path + ' -f n3', 'meta_owl.n3', filtr=filtr, comparator=ClickTestCase.n3_comparator)
        self.do_test(source_yaml_path + ' -f xsv', 'meta_error', error=click.exceptions.BadParameter)


if __name__ == '__main__':
    unittest.main()



import unittest

from rdflib import URIRef

from biolinkml.utils.curienamespace import CurieNamespace


class CurieNamespaceTestCase(unittest.TestCase):
    def test_basics(self):
        BFO = CurieNamespace('bfo', "http://purl.obolibrary.org/obo/BFO_")
        self.assertEqual(URIRef("http://purl.obolibrary.org/obo/BFO_test"), BFO.test)
        self.assertEqual("http://purl.obolibrary.org/obo/BFO_", BFO)
        self.assertEqual("bfo:test", BFO.curie('test'))
        self.assertEqual("bfo:", BFO.curie())

    @unittest.expectedFailure
    def test_curie_as_curie(self):
        """ "curie can't be a local name at the moment" """
        BFO = CurieNamespace('bfo', "http://purl.obolibrary.org/obo/BFO_")
        self.assertEqual("bfo:curie", BFO.curie)

if __name__ == '__main__':
    unittest.main()



import os
from contextlib import redirect_stdout
from io import StringIO
from typing import Union, TextIO, Optional, Set, List, Any, Callable, Dict

import click

from biolinkml.generators.yumlgen import YumlGenerator
from biolinkml.meta import SchemaDefinition, ClassDefinition, SlotDefinition, Element, ClassDefinitionName, \
    TypeDefinition
from biolinkml.utils.formatutils import camelcase, be, underscore, sfx
from biolinkml.utils.generator import Generator, shared_arguments
from biolinkml.utils.typereferences import References


class MarkdownGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.1.1"
    valid_formats = ["md"]
    visit_all_class_slots = False

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], **kwargs) -> None:
        super().__init__(schema, **kwargs)
        self.directory: Optional[str] = None
        self.image_directory: Optional[str] = None
        self.types_directory: Optional[str] = None
        self.noimages: bool = False
        self.gen_classes: Optional[Set[ClassDefinitionName]] = None
        self.gen_classes_neighborhood: Optional[References] = None
        self.BASE = None

    def visit_schema(self, directory: str = None, classes: Set[ClassDefinitionName] = None, image_dir: bool = False,
                     noimages: bool = False, **_) -> None:
        self.gen_classes = classes if classes else []
        for cls in self.gen_classes:
            if cls not in self.schema.classes:
                raise ValueError("Unknown class name: {cls}")
        if self.gen_classes:
            self.gen_classes_neighborhood = self.neighborhood(list(self.gen_classes))

        self.directory = directory
        if directory:
            os.makedirs(directory, exist_ok=True)
        elif image_dir:
            raise ValueError(f"Image directory can only be used with '-d' option")
        if image_dir:
            self.image_directory = os.path.join(directory, 'images')
            if not noimages:
                os.makedirs(self.image_directory, exist_ok=True)
        self.noimages = noimages
        self.types_directory = os.path.join(directory, 'types')
        os.makedirs(self.types_directory, exist_ok=True)

        with open(os.path.join(directory, 'index.md'), 'w') as ixfile:
            with redirect_stdout(ixfile):
                self.frontmatter(f"{self.schema.name.title()} schema")
                self.para(be(self.schema.description))

                self.header(3, 'Classes')
                for cls in sorted(self.schema.classes.values(), key=lambda c: c.name):
                    if not cls.is_a and not cls.mixin and self.is_secondary_ref(cls.name):
                        self.class_hier(cls)

                self.header(3, 'Mixins')
                for cls in sorted(self.schema.classes.values(), key=lambda c: c.name):
                    if cls.mixin and self.is_secondary_ref(cls.name):
                        self.class_hier(cls)

                self.header(3, 'Slots')
                for slot in sorted(self.schema.slots.values(), key=lambda s: s.name):
                    if not slot.is_a and self.is_secondary_ref(slot.name):
                        self.pred_hier(slot)

                self.header(3, 'Types')
                self.header(4, 'Built in')
                for builtin_name in sorted(self.synopsis.typebases.keys()):
                    self.bullet(f'**{builtin_name}**')
                self.header(4, 'Defined')
                for typ in sorted(self.schema.types.values(), key=lambda t: t.name):
                    if self.is_secondary_ref(typ.name):
                        if typ.typeof:
                            typ_typ = self.type_link(typ.typeof)
                        else:
                            typ_typ = f'**{typ.base}**'

                        self.bullet(self.type_link(typ, after_link=f' ({typ_typ})', use_desc=True))

    def visit_class(self, cls: ClassDefinition) -> bool:
        if self.gen_classes and cls.name not in self.gen_classes:
            return False
        with open(self.dir_path(cls), 'w') as clsfile:
            with redirect_stdout(clsfile):
                class_curi = self.namespaces.uri_or_curie_for(self.namespaces._base, camelcase(cls.name))
                class_uri = self.namespaces.uri_for(class_curi)
                self.element_header(cls, cls.name, class_curi, class_uri)
                print()
                if self.image_directory:
                    yg = YumlGenerator(self)
                    yg.serialize(classes=[cls.name], directory=self.image_directory, load_image=not self.noimages)
                    img_url = os.path.join('images', os.path.basename(yg.output_file_name))
                else:
                    yg = YumlGenerator(self)
                    img_url = yg.serialize(classes=[cls.name])\
                        .replace('?', '%3F').replace(' ', '%20').replace('|', '&#124;')

                print(f'![img]({img_url})')
                self.mappings(cls)

                if cls.is_a is not None:
                    self.header(2, 'Parents')
                    self.bullet(f' is_a: {self.class_link(cls.is_a, use_desc=True)}')
                if cls.mixins:
                    self.header(2, 'Uses Mixins')
                    for mixin in cls.mixins:
                        self.bullet(f' mixin: {self.class_link(mixin, use_desc=True)}')

                if cls.name in self.synopsis.isarefs:
                    self.header(2, 'Children')
                    for child in sorted(self.synopsis.isarefs[cls.name].classrefs):
                        self.bullet(f'{self.class_link(child, use_desc=True)}')

                if cls.name in self.synopsis.mixinrefs:
                    self.header(2, 'Mixin for')
                    for mixin in sorted(self.synopsis.mixinrefs[cls.name].classrefs):
                        self.bullet(f'{self.class_link(mixin, use_desc=True, after_link="(mixin)")}')

                if cls.name in self.synopsis.classrefs:
                    self.header(2, 'Referenced by class')
                    for sn in sorted(self.synopsis.classrefs[cls.name].slotrefs):
                        slot = self.schema.slots[sn]
                        if slot.range == cls.name:
                            self.bullet(f' **{self.class_link(slot.domain)}** '
                                        f'*{self.slot_link(slot, add_subset=False)}*{self.predicate_cardinality(slot)}  '
                                        f'**{self.class_type_link(slot.range)}**')

                self.header(2, 'Attributes')
                own_slots = [slot for slot in [self.schema.slots[sn]
                                               for sn in sorted(cls.slots)] if slot.owner == cls.name]
                if own_slots:
                    self.header(3, 'Own')
                    for slot in own_slots:
                        self.slot_field(cls, slot)

                for slot_owner in sorted({slot.owner for slot in [self.schema.slots[sn] for sn in cls.slots]
                                          if slot.owner != slot.name and slot.owner != cls.name}):
                    self.header(3, "Inherited from " + slot_owner + ':')
                    for owner_slot_name in self.schema.classes[slot_owner].slots:
                        owner_slot = self.schema.slots[owner_slot_name]
                        if owner_slot.owner == slot_owner:
                            self.slot_field(cls, owner_slot)

                domain_for_slots = [slot for slot in [self.schema.slots[sn]
                                                      for sn in sorted(cls.slots)] if slot.domain == cls.name]
                if domain_for_slots:
                    self.header(3, 'Domain for slot:')
                    for slot in domain_for_slots:
                        self.slot_field(cls, slot)

                self.element_properties(cls)

        return True

    def visit_type(self, typ: TypeDefinition) -> None:
        with open(self.dir_path(typ), 'w') as typefile:
            with redirect_stdout(typefile):
                full_path = sfx(self.namespaces._base) + (sfx(typ.imported_from) if typ.imported_from else '')
                type_curie = self.namespaces.uri_or_curie_for(full_path, camelcase(typ.name))
                type_uri = self.namespaces.uri_for(type_curie)
                self.element_header(typ, typ.name, type_curie, type_uri)

                print("|  |  |  |")
                print("| --- | --- | --- |")
                if typ.typeof:
                    print(f"| Parent type | | {self.class_type_link(typ.typeof)} |")
                print(f"| Root (builtin) type | | **{typ.base}** |")
                if typ.repr:
                    print(f"| Representation | | {typ.repr} |")
                self.element_properties(typ)

    def visit_class_slot(self, cls: ClassDefinition, aliased_slot_name: str, slot: SlotDefinition) -> None:
        with open(self.dir_path(slot), 'w') as slotfile:
            with redirect_stdout(slotfile):
                slot_curie = self.namespaces.uri_or_curie_for(self.namespaces._base, underscore(slot.name))
                slot_uri = self.namespaces.uri_for(slot_curie)
                self.element_header(slot,aliased_slot_name, slot_curie, slot_uri)
                self.mappings(slot)

                self.header(2, 'Domain and Range')
                print(f'{self.class_link(slot.domain)} ->{self.predicate_cardinality(slot)} '
                      f'{self.class_type_link(slot.range)}')

                self.header(2, 'Parents')
                if slot.is_a:
                    self.bullet(f' is_a: {self.slot_link(slot.is_a)}')

                self.header(2, 'Children')
                if slot.name in sorted(self.synopsis.isarefs):
                    for child in sorted(self.synopsis.isarefs[slot.name].slotrefs):
                        self.bullet(f' {self.slot_link(child)}')

                self.header(2, 'Used by')
                if slot.name in sorted(self.synopsis.slotrefs):
                    for rc in sorted(self.synopsis.slotrefs[slot.name].classrefs):
                        self.bullet(f'{self.class_link(rc)}')
                if aliased_slot_name == 'relation':
                    if slot.subproperty_of:
                        self.bullet(f' reifies: {self.slot_link(slot.subproperty_of)}')
                self.element_properties(slot)

    def element_header(self, obj: Element, name: str, curie: str, uri: str) -> None:
        self.frontmatter(f"Type: {obj.name}" + (f" _(deprecated)_" if obj.deprecated else ""))
        self.para(be(obj.description))
        print(f'URI: [{curie}]({uri})')
        print()

    def element_properties(self, obj: Element) -> None:
        def identity(e: Any) -> Any:
            return e

        def prop_list(title: str, entries: Union[List, Dict], formatter: Optional[Callable[[Element], str]] = None) \
                -> None:
            if formatter is None:
                formatter = identity
            if entries:
                if isinstance(entries, Dict):
                    entries = list(entries.values())
                print(f"| **{title}:** | | {formatter(entries[0])} |")
                for entry in entries[1:]:
                    print(f"|  | | {formatter(entry)} |")

        attributes = StringIO()
        with redirect_stdout(attributes):
            prop_list('Aliases', obj.aliases)
            prop_list('Local names', obj.local_names, lambda e: f"{e.local_name_value} ({e.local_name_source})")
            prop_list('Mappings', obj.mappings)
            prop_list('Alt Descriptions', obj.alt_descriptions, lambda e: f"{e.description} ({e.source})")
            # todos
            # notes
            prop_list('Comments', obj.comments)
            prop_list('Examples', obj.examples)
            prop_list('In Subsets', obj.in_subset)
            # from_schema
            # imported_from
            prop_list('See also', obj.see_also)
            #       - exact mappings
            #       - close mappings
            #       - related mappings
            #       - deprecated element has exact replacement
            #       - deprecated element has possible replacement
        if attributes.getvalue():
            self.header(2, 'Other properties')
            print("|  |  |  |")
            print("| --- | --- | --- |")
            print(attributes.getvalue())

    def class_hier(self, cls: ClassDefinition, level=0) -> None:
        self.bullet(self.class_link(cls, use_desc=True), level)
        if cls.name in sorted(self.synopsis.isarefs):
            for child in sorted(self.synopsis.isarefs[cls.name].classrefs):
                self.class_hier(self.schema.classes[child], level+1)
        
    def pred_hier(self, slot: SlotDefinition, level=0) -> None:
        self.bullet(self.slot_link(slot, use_desc=True), level)
        if slot.name in sorted(self.synopsis.isarefs):
            for child in sorted(self.synopsis.isarefs[slot.name].slotrefs):
                self.pred_hier(self.schema.slots[child], level+1)

    def dir_path(self, obj: Union[ClassDefinition, SlotDefinition, TypeDefinition]) -> str:
        filename = self.formatted_element_name(obj) if isinstance(obj, ClassDefinition) \
            else underscore(obj.name) if isinstance(obj, SlotDefinition) \
            else camelcase(obj.name)
        subdir = '/types' if isinstance(obj, TypeDefinition) else ''
        return f'{self.directory}{subdir}/{filename}.md'

    def mappings(self, obj: [SlotDefinition, ClassDefinition]) -> None:
        # TODO: get rid of this?
        # self.header(2, 'Mappings')
        # for mapping in obj.mappings:
        #     self.bullet(f"{self.xlink(mapping)} {self.to_uri(mapping)}")
        # if obj.subclass_of:
        #     self.bullet(self.xlink(obj.subclass_of))
        pass

    def is_secondary_ref(self, en: str) -> bool:
        """ Determine whether 'en' is the name of something in the neighborhood of the requested classes

        @param en: element name
        @return: True if 'en' is the name of a slot, class or type in the immediate neighborhood of of what we are
        building
        """
        if not self.gen_classes:
            return True
        elif en in self.schema.classes:
            return en in self.gen_classes_neighborhood.classrefs
        elif en in self.schema.slots:
            return en in self.gen_classes_neighborhood.slotrefs
        elif en in self.schema.types:
            return en in self.gen_classes_neighborhood.typerefs
        else:
            return True

    def slot_field(self, cls: ClassDefinition, slot: SlotDefinition) -> None:
        self.bullet(f'{self.slot_link(slot)}{self.predicate_cardinality(slot)}')
        if slot.description:
            self.bullet(f'Description: {slot.description}', level=1)
        self.bullet(f'range: {self.class_type_link(slot.range)}', level=1)
        # if slot.subproperty_of:
        #     self.bullet(f'edge label: {self.slot_link(slot.subproperty_of)}', level=1)
        for example in slot.examples:
            self.bullet(f'Example: {example.value} {example.description}', level=1)
        if slot.name not in self.own_slot_names(cls):
            self.bullet(f'inherited from: {self.class_link(slot.domain)}', level=1)
        if slot.in_subset:
            ssl = ','.join(slot.in_subset)
            self.bullet(f'in subsets: ({ssl})', level=1)

    def to_uri(self, uri_or_curie: str) -> str:
        """ Return the URI for the slot if known """
        if ':/' in uri_or_curie:
            return uri_or_curie
        elif self.namespaces.prefix_for(uri_or_curie) in self.namespaces:
            return self.namespaces.uri_for(uri_or_curie)
        else:
            return "*(Unknown namespace)*"

    # --
    # FORMATTING
    # --
    @staticmethod
    def predicate_cardinality(slot: SlotDefinition) -> str:
        """ Emit cardinality for a suffix on a predicate"""
        if slot.multivalued:
            card_str = '1..*' if slot.required else '0..*'
        else:
            card_str = 'REQ' if slot.required else 'OPT'
        return f"  <sub>{card_str}</sub>"

    @staticmethod
    def range_cardinality(slot: SlotDefinition) -> str:
        """ Emits cardinality decorator at end of type """
        if slot.multivalued:
            card_str = '1..*' if slot.required else '0..*'
        else:
            card_str = '1..1' if slot.required else '0..1'
        return f"  <sub><b>{card_str}</b></sub>"

    @staticmethod
    def anchor(id_: str) -> None:
        print(f'<a name="{id_}">', end='')

    @staticmethod
    def anchorend() -> None:
        print('</a>')

    @staticmethod
    def header(level: int, txt: str) -> None:
        print(f'\n{"#" * level} {txt}\n')

    @staticmethod
    def para(txt: str) -> None:
        print(f'\n{txt}\n')

    @staticmethod
    def bullet(txt: str, level=0) -> None:
        print(f'{"   " * level} * {txt}')

    def frontmatter(self, thingtype: str, layout='default') -> None:
        self.header(1, thingtype)
        # print(f'---\nlayout: {layout}\n---\n')

    def bbin(self, obj: Element) -> str:
        """ Boldify built in types

        @param obj: object name or id
        @return:
        """
        return obj.name if isinstance(obj, Element ) else f'**{obj}**' if obj in self.synopsis.typebases else obj

    def desc_for(self, obj: Element, doing_descs: bool) -> str:
        """ Return a description for object if it is unique (different than its parent)

        @param obj: object to be described
        @param doing_descs: If false, always return an empty string
        @return: text or empty string
        """
        if obj.description and doing_descs:
            if isinstance(obj, SlotDefinition) and obj.is_a:
                parent = self.schema.slots[obj.is_a]
            elif isinstance(obj, ClassDefinition) and obj.is_a:
                parent = self.schema.classes[obj.is_a]
            else:
                parent = None
            return '' if parent and obj.description == parent.description else obj.description
        return ''

    def _link(self, obj: Optional[Element], *, after_link: str = None, use_desc: bool=False,
             add_subset: bool=True) -> str:
        """ Create a link to ref if appropriate.

        @param ref: the name or value of a class, slot, type or the name of a built in type.
        @param after_link: Text to put between link and description
        @param use_desc: True means append a description after the link if available
        @param add_subset: True means add any subset information that is available
        @return:
        """
        nl = '\n'
        if obj is None or not self.is_secondary_ref(obj.name):
            return self.bbin(obj)
        if isinstance(obj, SlotDefinition):
            link_name = ((be(obj.domain) + '➞') if obj.alias else '') + self.aliased_slot_name(obj)
            link_ref = underscore(obj.name)
        elif isinstance(obj, TypeDefinition):
            link_name = camelcase(obj.name)
            link_ref = f"types/{link_name}"
        elif isinstance(obj, ClassDefinition):
            link_name = camelcase(obj.name)
            link_ref = camelcase(link_name)
        else:
            link_name = obj.name
            link_ref = link_name
        desc = self.desc_for(obj, use_desc)
        return f'[{link_name}]' \
               f'({link_ref}.{self.format})' + \
                 (f' {after_link} ' if after_link else '') + (f' - {desc.split(nl)[0]}' if desc else '')

    def type_link(self, ref: Optional[Union[str, TypeDefinition]], *, after_link: str = None, use_desc: bool=False,
             add_subset: bool=True) -> str:
        return self._link(self.schema.types[ref] if isinstance(ref, str) else ref, after_link=after_link,
                         use_desc=use_desc, add_subset=add_subset)

    def slot_link(self, ref: Optional[Union[str, SlotDefinition]], *, after_link: str = None, use_desc: bool=False,
             add_subset: bool=True) -> str:
        return self._link(self.schema.slots[ref] if isinstance(ref, str) else ref, after_link=after_link,
                         use_desc=use_desc, add_subset=add_subset)

    def class_link(self, ref: Optional[Union[str, ClassDefinition]], *, after_link: str = None, use_desc: bool=False,
             add_subset: bool=True) -> str:
        return self._link(self.schema.classes[ref] if isinstance(ref, str) else ref, after_link=after_link,
                         use_desc=use_desc, add_subset=add_subset)

    def class_type_link(self, ref: Optional[Union[str, ClassDefinition, TypeDefinition]], *, after_link: str = None,
                        use_desc: bool=False, add_subset: bool=True) -> str:
        if isinstance(ref, ClassDefinition):
            return self.class_link(ref, after_link=after_link, use_desc=use_desc, add_subset=add_subset)
        elif isinstance(ref, TypeDefinition):
            return self.type_link(ref, after_link=after_link, use_desc=use_desc, add_subset=add_subset)
        elif ref in self.schema.classes:
            return self.class_link(ref, after_link=after_link, use_desc=use_desc, add_subset=add_subset)
        else:
            return self.type_link(ref, after_link=after_link, use_desc=use_desc, add_subset=add_subset)

    def xlink(self, id_: str) -> str:
        return f'[{id_}]({self.id_to_url(id_)})'


    @staticmethod
    def id_to_url(id_: str) -> str:
        uri = id_
        if ':' in id_:
            # TODO! use PC
            if id_.startswith('SIO:'):
                uri = id_.replace('SIO:', 'http://semanticscience.org/resource/SIO_')
            elif id_.startswith('HGNC:'):
                uri = 'https://monarchinitiative.org/gene/' + id_
            else:
                frag = id_.replace(':', '_')
                base = 'http://purl.obolibrary.org/obo/'
                uri = base+frag
        return uri


@shared_arguments(MarkdownGenerator)
@click.command()
@click.option("--dir", "-d", help="Output directory")
@click.option("--classes", "-c", default=None, multiple=True, help="Class(es) to emit")
@click.option("--img", "-i",  is_flag=True, help="Download YUML images to 'image' directory")
@click.option("--noimages", is_flag=True, help="Do not (re-)generate images")
def cli(yamlfile, dir, img, **kwargs):
    """ Generate markdown documentation of a biolink model """
    MarkdownGenerator(yamlfile, **kwargs).serialize(directory=dir, image_dir=img, **kwargs)



import os
import re
import unittest
from typing import Optional, Callable, Union, Dict

from rdflib import Graph, Namespace, OWL

from biolinkml import METAMODEL_NAMESPACE, MODULE_DIR
from biolinkml.utils.context_utils import parse_import_map
from biolinkml.utils.generator import Generator
from tests import sourcedir, USE_LOCAL_IMPORT_MAP, DEFAULT_LOG_LEVEL
from tests.test_scripts.clicktestcase import ClickTestCase
from tests.utils.compare_directories import are_dir_trees_equal
from tests.utils.dirutils import make_and_clear_directory
from tests.utils.rdf_comparator import compare_rdf, to_graph

BIOLINK_NS = Namespace("https://w3id.org/biolink/vocab/")

# ShEx validation of the biolink model takes a loooong time, so we only do it on rare occasions
DO_SHEX_VALIDATION = False

# Normally we import mappings and types directly from the URL, but for testing we use local maps
BIOLINK_IMPORT_MAP_FNAME = os.path.join(sourcedir, 'local_import_map.json' if USE_LOCAL_IMPORT_MAP else 'biolink_import_map.json')
BIOLINK_IMPORT_MAP = parse_import_map(BIOLINK_IMPORT_MAP_FNAME,
                                      os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..')))



class GeneratorTestCase(unittest.TestCase):
    source_path: str = None             # Path to expected output
    target_path: str = None             # Path to actual output
    model_path: str = None              # Path for yaml
    model_name: str = None              # yaml name (sans '.yaml')
    output_name: str = None             # If different than model name
    importmap: Dict[str, str] = None   # Location of the import mappings if any

    def verify_paths(self) -> None:
        assert self.source_path and self.target_path and self.model_path and self.model_name, "Paths must be set"

    @staticmethod
    def _print_triples(g: Graph):
        """ Pretty print the contents of g, removing the prefix declarations """
        g.bind('BIOLINK', BIOLINK_NS)
        g.bind('meta', METAMODEL_NAMESPACE)
        g.bind('owl', OWL)
        g_text = re.sub(r'@prefix.*\n', '', g.serialize(format="turtle").decode())
        print(g_text)

    def _default_comparator(self, old_data: str, new_data: str, new_file: str, msg: Optional[str] = None,
                            filtr: Optional[Callable[[str], str]] = None) -> None:
        """ Simple file comparator.  Compare old to new and, if they don't match, save an image of new in
        file_name_for_actual and raise an error

        :param old_data: Expected data
        :param new_data: Actual data
        :param new_file: Save actual data here if mismatch
        :param msg: If present add to the assert message
        :param filtr: data filter
        :return:
        """
        self.maxDiff = None
        old_data_filtered = filtr(old_data) if filtr is not None else old_data
        new_data_filtered = filtr(new_data) if filtr is not None else new_data
        if old_data_filtered != new_data_filtered:
            if msg:
                print(msg)
            with open(new_file, 'w') as newf:
                newf.write(new_data)
            if len(new_data) > 20000:
                print(ClickTestCase.closein_comparison(old_data_filtered, new_data_filtered))
            self.maxDiff = None
            self.assertEqual(old_data_filtered, new_data_filtered)

    def rdf_comparator(self, expected_rdf: Union[Graph, str], actual_rdf: Union[Graph, str],
                       file_name_for_actual: Optional[str] = None, msg: Optional[str] = None,
                       filtr: Optional[Callable[[str], str]] = None) -> None:
        """
        RDF comparator.  Compare two graphs and, if they don't match, save a turtle image of new_data in
        new_file and raise an error
        :param expected_rdf: Turtle representation of expected RDF
        :param actual_rdf: Turtle representation of actual RDF
        :param file_name_for_actual: Save actual RDF here if mismatch
        :param msg: If present, add to assert message
        :param filtr: data filter
        :return:
        """
        error_msg = compare_rdf(expected_rdf, actual_rdf)
        if error_msg:
            if file_name_for_actual:
                to_graph(actual_rdf).serialize(file_name_for_actual, format="turtle")
                print(f"***** New graph saved in {file_name_for_actual} *****")
            print(error_msg)
            self.fail("RDF file mismatch" if not msg else msg)

    def single_file_generator(self, suffix: str, gen: type(Generator), *,
                              format: Optional[str] = None,
                              generator_args: Optional[dict] = None,
                              serialize_args: Optional[dict] = None,
                              filtr: Optional[Callable[[str], str]] = None,
                              comparator: Callable[[type(unittest.TestCase), str, str, str, Optional[Callable[[str], str]]], None] = None,
                              preserve_metadata: bool = False,
                              fail_if_expected_missing: bool = True) -> str:
        """ Invoke generator specified in gen

        :param suffix: File suffix (without '.')
        :param gen: Generator to invoke
        :param format: Generator format argument
        :param generator_args: Additional arguments to the generator
        :param serialize_args: Arguments to serializer.
        :param filtr: Filter to remove metadata specific info from the output.  Default: identity
        :param comparator: Comparison method to use.  Default: GeneratorTestCase._default_comparator
        :param preserve_metadata: True means metadata is to preserved in old_file
        :param fail_if_expected_missing: True means return error message rather than fail
        :return: Empty string if success else error message if fail_if_expected_missing is false
        """
        self.verify_paths()
        if serialize_args is None:
            serialize_args = {}
        if generator_args is None:
            generator_args = {}
        if format:
            generator_args["format"] = format
        if filtr is None:
            def filtr(s): return s
        if comparator is None:
            comparator = GeneratorTestCase._default_comparator
        if self.importmap is not None and 'importmap' not in generator_args:
            generator_args['importmap'] = self.importmap
        output_base = self.output_name if self.output_name is not None else self.model_name
        old_file = os.path.join(self.source_path, output_base + ('.' if output_base else '') + suffix)
        new_file = os.path.join(self.target_path, output_base + ('.' if output_base else '') + suffix)
        message = \
            f"\n***** Expected output is in {os.path.relpath(old_file, MODULE_DIR)} " \
            f"and actual is in {os.path.relpath(new_file, MODULE_DIR)} *****\n" \
            f"***** Remove {os.path.relpath(old_file, MODULE_DIR)} and re-run to update"
        yaml_file = os.path.join(self.model_path, self.model_name + '.yaml')
        if os.path.exists(new_file):
            os.remove(new_file)

        generator_args['log_level'] = DEFAULT_LOG_LEVEL
        new_data = str(gen(yaml_file, **generator_args).serialize(**serialize_args))
        if not os.path.exists(old_file):
            with open(old_file, 'w') as oldf:
                oldf.write(new_data if preserve_metadata else filtr(new_data))
            msg = f"Created {old_file} - run again"
            if fail_if_expected_missing:
                self.fail(msg)
            return msg + '\n'

        with open(old_file) as oldf:
            old_data = filtr(oldf.read())
        comparator(self, old_data, new_data, new_file, message, filtr=filtr)
        return ''

    def directory_generator(self, dirname: str, gen: type(Generator), gen_args: Optional[dict] = None,
                            serialize_args: Optional[dict] = None, skip_compare_step: bool = False) -> None:
        """
        Generate an output directory using the appropriate command and then compare the target with the source
        :param dirname: name of output directory (e.g. gengraphviz)
        :param gen: generator to use
        :param gen_args: arguments to the generator constructor
        :param serialize_args: arguments to the generator serializer
        :param skip_compare_step: True means just generate -- don't compare
        """
        self.verify_paths()
        if gen_args is None:
            gen_args = {}
        if serialize_args is None:
            serialize_args = {}
        if self.importmap is not None and 'importmap' not in gen_args:
            gen_args['importmap'] = self.importmap
        source_dir = os.path.join(self.source_path, dirname)
        if not os.path.exists(source_dir) or skip_compare_step:
            make_and_clear_directory(source_dir)
            target_dir = source_dir
        else:
            target_dir = os.path.join(self.target_path, dirname)
            make_and_clear_directory(target_dir)
        yaml_file = os.path.join(self.model_path, self.model_name + '.yaml')
        gen(yaml_file, **gen_args).serialize(directory=target_dir, **serialize_args)
        if source_dir == target_dir:
            if skip_compare_step:
                print(f"Output generated in {dirname} -- you may want to check it manually")
                return
            else:
                self.fail(f"{dirname} created - rerun test")
        diffs = are_dir_trees_equal(target_dir, source_dir)
        if diffs:
            print(diffs)
            self.assertTrue(False, f"{dirname} mismatch")
        make_and_clear_directory(target_dir)



import os
import unittest

from biolinkml.generators.markdowngen import cli
from tests import source_yaml_path
from tests.test_scripts.clicktestcase import ClickTestCase


class GenMarkdownTestCase(ClickTestCase):
    testdir = "genmarkdown"
    click_ep = cli
    prog_name = "gen-markdown"

    def test_help(self):
        self.do_test("--help", 'help')

    def test_meta(self):
        outdir = self.temp_directory('meta')
        self.do_test(source_yaml_path + f" -d {outdir}", dirbase='meta')

    def test_issue_2(self):
        outdir = self.temp_directory('issue2')
        self.do_test(source_yaml_path + f" -d {outdir} -c example -i ", dirbase='issue2')
        ex_file = os.path.join(outdir, 'images', 'Example.svg')
        self.assertTrue(os.path.exists(ex_file), f"Filed to create {ex_file}")
        self.assertFalse(os.path.exists(os.path.join(outdir, 'abstract.md')))


if __name__ == '__main__':
    unittest.main()



from dataclasses import dataclass
from typing import Set, cast

from biolinkml.meta import ClassDefinitionName, SlotDefinitionName, TypeDefinitionName, SubsetDefinitionName, ElementName
from biolinkml.utils.metamodelcore import empty_set


@dataclass(repr=False, frozen=True)
class RefType:
    name: str

    def __repr__(self):
        return self.name


ClassType = RefType('Class')
TypeType = RefType('Type')
SlotType = RefType('Slot')
SubsetType = RefType('Subset')


@dataclass
class References:
    """
    Summary of references to a given class. The reference class is the key to the dictionary carrying classrefs
    """
    classrefs: Set[ClassDefinitionName] = empty_set()     # Refs of type class
    slotrefs: Set[SlotDefinitionName] = empty_set()       # Refs of type slot
    typerefs: Set[TypeDefinitionName] = empty_set()       # Refs of type type
    subsetrefs: Set[SubsetDefinitionName] = empty_set()   # Refs of type subset

    def addref(self, fromtype: RefType, fromname: ElementName) -> None:
        if fromtype is ClassType:
            self.classrefs.add(cast(ClassDefinitionName, fromname))
        elif fromtype is TypeType:
            self.typerefs.add(cast(TypeDefinitionName, fromname))
        elif fromtype is SlotType:
            self.slotrefs.add(cast(SlotDefinitionName, fromname))
        elif fromtype is SubsetType:
            self.subsetrefs.add(cast(SubsetDefinitionName, fromname))
        else:
            raise TypeError(f"Unknown typ: {fromtype}")

    def update(self, other: "References") -> None:
        self.classrefs.update(other.classrefs)
        self.slotrefs.update(other.slotrefs)
        self.typerefs.update(other.typerefs)
        self.subsetrefs.union(other.subsetrefs)

    def __bool__(self):
        return bool(self.classrefs) or bool(self.slotrefs) or bool(self.typerefs) or bool(self.subsetrefs)



import os


def make_and_clear_directory(dirbase: str) -> None:
    """ Make dirbase if necessary and then clear generated files """
    import shutil

    safety_file = os.path.join(dirbase, "generated")
    if os.path.exists(dirbase):
        if not os.path.exists(safety_file):
            raise FileNotFoundError("'generated' guard file not found in {}".format(safety_file))
        shutil.rmtree(dirbase)
    os.makedirs(dirbase)
    with open(os.path.join(dirbase, "generated"), "w") as f:
        f.write("Generated for safety.  Directory will not be cleared if this file is not present")



from types import ModuleType

from jsonasobj import as_json, loads
from rdflib import Graph

from biolinkml.generators.jsonldcontextgen import ContextGenerator
from biolinkml.generators.pythongen import PythonGenerator
from biolinkml.generators.shexgen import ShExGenerator
from biolinkml.generators.yumlgen import YumlGenerator
from biolinkml.utils.yamlutils import as_json_object as yaml_to_json

yaml = """
id: http://example.org/sample/example2
name: inheritence
prefixes:
    foaf: http://xmlns.com/foaf/0.1/
    ex: http://example.org/model/
    xsd: http://www.w3.org/2001/XMLSchema#

default_prefix: ex

default_curi_maps:
    - semweb_context

default_range: string

types:
    string:
        base: str
        uri: xsd:string
    int:
        base: int
        uri: xsd:integer
    boolean:
        base: Bool
        uri: xsd:boolean


classes:
    root:
        description: an empty class
        
    children:
        description: an identified class
        is_a: root
        slots:
            - root_id
            
    child_1:
        description: first child
        is_a: children
        
    child_2:
        description: second child
        is_a: children
        slots:
            - description
            
    child_2_1:
        description: grand child with a parent slot
        is_a: child_2
        
    child_2_2:
        description: grand child with parent and own slot
        is_a: child_2
        slots:
            - angry
            
slots:
    root_id:
        description: Unique identifier 
        identifier: true
        
    description:
        description: Text description of class
        required: true
        
    angry:
        description: angry grandchild
        range: boolean
        required: true
"""

print(f'<img src="{YumlGenerator(yaml).serialize()}"/>')
print(f'\n-----\n{YumlGenerator(yaml).serialize()}\n')

shex = ShExGenerator(yaml).serialize(collections=False)
print(shex)

# # Generate a person
# joe_smith = module.Person(id="42", last_name="smith", first_name=['Joe', 'Bob'], age=43)
# print(joe_smith)
#
# # Add the context and turn it into RDF
# jsonld = as_json_object(yaml_to_json(joe_smith, cntxt))
# print(jsonld)
# g = Graph()
# g.parse(data=jsonld, format="json-ld")
# print(g.serialize(format="turtle").decode())
#
# from pyshex.evaluate import evaluate
#
# r = evaluate(g, shex,
#              start="http://example.org/sample/example1/Person",
#              focus="http://example.org/context/42")
# print("Conforms" if r[0] else r[1])



import os
import unittest
from types import ModuleType

from jsonasobj import as_json

from biolinkml.generators.jsonldcontextgen import ContextGenerator
from biolinkml.generators.pythongen import PythonGenerator
from biolinkml.utils.yamlutils import as_rdf
from tests.test_issues import sourcedir


class Issue121TestCase(unittest.TestCase):

    def header(self, txt: str) -> str:
        return '\n' + ("=" * 20) + f" {txt} " + ("=" * 20)

    def test_issue_121(self):
        """ Make sure that types are generated as part of the output """
        yaml_fname = os.path.join(sourcedir, 'issue_121.yaml')
        python = PythonGenerator(yaml_fname).serialize()
        print(self.header("Python"))
        print(python)

        has_includes = False
        for line in python.split("\n"):
            if line.startswith("from includes.types"):
                assert line == "from includes.types import String"
                has_includes = True
        assert has_includes

        spec = compile(python, 'test', 'exec')
        module = ModuleType('test')
        exec(spec, module.__dict__)
        example = module.Biosample(depth="test")
        assert hasattr(example, "depth")
        assert example.depth == "test"
        # JSON Representation
        print(self.header("JSON"))
        print(as_json(example))
        example2 = module.ImportedClass()


if __name__ == '__main__':
    unittest.main()



import os
import unittest

# This has to occur post ClickTestCase
from types import ModuleType

import click


from biolinkml.generators.pythongen import cli, PythonGenerator
from tests import sourcedir, targetdir, source_yaml_path
from tests.test_scripts.clicktestcase import ClickTestCase
from tests.utils.metadata_filters import metadata_filter


class GenPythonTestCase(ClickTestCase):
    testdir = "genpython"
    click_ep = cli
    prog_name = "gen-py-classes"

    def gen_and_comp_python(self, base: str) -> None:
        """ Generate yaml_file into python_file and compare it against master_file  """
        yaml_path = os.path.abspath(os.path.join(sourcedir, f"{base}.yaml"))
        target_path = os.path.join(targetdir, f'{base}.py')
        master_path = os.path.join(sourcedir, f'{base}.py')

        pydata = str(PythonGenerator(yaml_path, "py", emit_metadata=False).serialize())
        newdat = metadata_filter(pydata)

        # If the master for comparison doesn't exist, create it
        if not os.path.exists(master_path):
            with open(master_path, 'w') as mf:
                mf.write(newdat)
            self.assertFalse(True, f"Regenerated {os.path.basename(master_path)} - rerun to complete test")

        # Compare the current master with what we've generated
        with open(master_path) as oldf:
            olddat = oldf.read()
        if olddat != newdat:
            # Save the old if we're different
            with open(target_path, 'w') as newf:
                newf.write(pydata)
            self.maxDiff = None
            if olddat != newdat:
                print(f"Data file: {master_path}")
            self.assertEqual(olddat, newdat)

        # Make sure the python is valid
        spec = compile(pydata, 'test', 'exec')
        module = ModuleType('test')
        exec(spec, module.__dict__)

        if os.path.exists(target_path):
            os.remove(target_path)

    def test_help(self):
        self.do_test("--help", 'help')

    def test_meta(self):
        self.maxDiff = None
        self.do_test(source_yaml_path, 'meta.py', filtr=metadata_filter)
        self.do_test(source_yaml_path + ' -f py', 'meta.py', filtr=metadata_filter)
        self.do_test(source_yaml_path + ' -f xsv', 'meta_error', error=click.exceptions.BadParameter)

    def test_head(self):
        """ Validate the head/nohead parameter """
        yaml = '''id: "https://w3id.org/biolink/metamodel"
description: Metamodel for biolink schema
license: https://creativecommons.org/publicdomain/zero/1.0/
version: 0.4.0
default_range: string
prefixes:
    xsd: http://www.w3.org/2001/XMLSchema#
types:
   string:
      base: str
      uri: xsd:string'''
        output = PythonGenerator(yaml, "py", emit_metadata=True).serialize()
        self.assertTrue(output.startswith(f'# Auto generated from None by pythongen.py version: '
                                          f'{PythonGenerator.generatorversion}'))
        output = PythonGenerator(yaml, "py", emit_metadata=False).serialize()
        self.assertTrue(output.startswith('\n# id: https://w3id.org/biolink/metamodel'))

    def test_multi_id(self):
        """ Test the multi-identifier error """
        self.gen_and_comp_python('multi_id')

    @unittest.skipIf(True, "See https://github.com/biolink/biolinkml/issues/141")
    def test_timepoint(self):
        """ Test an issue with the biolink-model timepoint rendering """
        self.gen_and_comp_python('timepoint')

    def test_type_inheritence(self):
        """ Make sure that typeof's get represented correctly """
        self.gen_and_comp_python('testtypes')

    def test_inherited_identifiers(self):
        self.gen_and_comp_python('inheritedid')

    # This still needs to be fixed
    @unittest.expectedFailure
    def test_ordering(self):
        self.gen_and_comp_python('ordering')

    def test_default_namespace(self):
        """ Test that curie_for replaces '@default' with a blank """
        self.gen_and_comp_python('default_namespace')


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from jsonasobj import loads
from rdflib import Namespace

from biolinkml import LOCAL_TYPES_YAML_FILE
from biolinkml.generators.shexgen import ShExGenerator


METATYPE = Namespace("https://w3id.org/biolink/biolinkml/type/")


class UriTypeTestCase(unittest.TestCase):
    def test_uri_type(self):
        """ URI datatype should map to ShEx URI instead of NONLITERAL """
        shex = loads(ShExGenerator(LOCAL_TYPES_YAML_FILE, format='json').serialize())
        uri_shape = [s for s in shex.shapes if s.id == str(METATYPE.Uri)]
        self.assertEqual(1, len(uri_shape))
        self.assertEqual('iri', uri_shape[0].nodeKind)


if __name__ == '__main__':
    unittest.main()



import os
from typing import Union, TextIO

import click
from jsonasobj import JsonObj, as_json

from biolinkml.utils.generator import Generator, shared_arguments
from biolinkml.meta import SchemaDefinition, ClassDefinition, SlotDefinition
from biolinkml.utils.formatutils import camelcase, be, underscore


class JsonSchemaGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.0.2"
    valid_formats = ["json"]
    visit_all_class_slots = True

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], top_class : str = None, **kwargs) -> None:
        super().__init__(schema, **kwargs)
        self.schemaobj: JsonObj = None
        self.clsobj: JsonObj = None
        self.inline = False
        self.topCls = top_class  ## JSON object is one instance of this
        self.entryProperties = {}
        # JSON-Schema does not have inheritance,
        # so we duplicate slots from inherited parents and mixins
        self.visit_all_slots = True

    def visit_schema(self, inline: bool=False, **kwargs) -> None:
        self.inline = inline
        self.schemaobj = JsonObj(title=self.schema.name,
                                 type="object",
                                 properties={},
                                 definitions=JsonObj())
        for p,c in self.entryProperties.items():
            self.schemaobj['properties'][p] = {
                'type': "array",
                 'items': { '$ref': f"#/definitions/{camelcase(c)}"}}
        self.schemaobj['$schema'] = "http://json-schema.org/draft-07/schema#"
        self.schemaobj['$id'] = self.schema.id

    def end_schema(self, **_) -> None:
        print(as_json(self.schemaobj, sort_keys=True))

    def visit_class(self, cls: ClassDefinition) -> bool:
        if cls.abstract:
            return False
        self.clsobj = JsonObj(title=camelcase(cls.name),
                              type='object',
                              properties=JsonObj(),
                              required=[],
                              additionalProperties=False,
                              description=be(cls.description))
        return True

    def end_class(self, cls: ClassDefinition) -> None:
        self.schemaobj.definitions[camelcase(cls.name)] = self.clsobj

    def visit_class_slot(self, cls: ClassDefinition, aliased_slot_name: str, slot: SlotDefinition) -> None:
        if slot.range in self.schema.classes and slot.inlined:
            rng = f"#/definitions/{camelcase(slot.range)}"
        elif slot.range in self.schema.types:
            rng = self.schema.types[slot.range].base
        else:
            # note we assume string for non-lined complex objects
            rng = "string"

        # translate to json-schema builtins
        if rng == 'int':
            rng = 'integer'
        elif rng == 'Bool':
            rng = 'boolean'
        elif rng == 'str':
            rng = 'string'
        elif rng == 'float' or rng == 'double':
            rng = 'number'
        elif not rng.startswith('#'):
            # URIorCURIE, etc
            rng = 'string'

        if slot.inlined:
            # If inline we have to include redefined slots
            ref = JsonObj()
            ref['$ref'] = rng
            if slot.multivalued:
                prop = JsonObj(type="array", items=ref)
            else:
                prop = ref
        else:
            if slot.multivalued:
                prop = JsonObj(type="array", items={'type':rng})
            else:
                prop = JsonObj(type=rng)
        if slot.description:
            prop.description = slot.description
        if slot.required:
            self.clsobj.required.append(underscore(aliased_slot_name))

        self.clsobj.properties[underscore(aliased_slot_name)] = prop
        if self.topCls is not None and camelcase(self.topCls) == camelcase(cls.name):
            self.schemaobj.properties[underscore(aliased_slot_name)] = prop


@shared_arguments(JsonSchemaGenerator)
@click.command()
@click.option("-i", "--inline", is_flag=True, help="""
Generate references to types rather than inlining them.
Note that declaring a slot as inlined: true will always inline the class
""")
@click.option("-t", "--top-class", help="""
Top level class; slots of this class will become top level properties in the json-schema
""")
def cli(yamlfile, **kwargs):
    """ Generate JSON Schema representation of a biolink model """
    print(JsonSchemaGenerator(yamlfile, **kwargs).serialize(**kwargs))



"""Generate ShEx definition of a model

"""
import os
from typing import Union, TextIO, Optional, List

import click
from ShExJSG import ShExC
from ShExJSG.SchemaWithContext import Schema
from ShExJSG.ShExJ import Shape, IRIREF, EachOf, TripleConstraint, NodeConstraint, ShapeOr
from jsonasobj import as_json
from rdflib import Graph, OWL, RDF, Namespace, XSD

from biolinkml import METAMODEL_NAMESPACE_NAME, METAMODEL_NAMESPACE
from biolinkml.meta import SchemaDefinition, ClassDefinition, SlotDefinition, SlotDefinitionName, ElementName, \
    TypeDefinition
from includes.types import SHEX
from biolinkml.utils.formatutils import camelcase, sfx
from biolinkml.utils.generator import Generator, shared_arguments
from biolinkml.utils.metamodelcore import URIorCURIE


class ShExGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.0.2"
    valid_formats = ['shex', 'json', 'rdf']
    visit_all_class_slots = False

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], **args) -> None:
        super().__init__(schema, **args)
        self.shex: Schema = Schema()                # ShEx Schema being generated
        self.shapes = []
        self.shape: Optional[Shape] = None                    # Current shape being defined
        self.list_shapes: List[IRIREF] = []         # Shapes that have been defined as lists

        if METAMODEL_NAMESPACE_NAME not in self.namespaces:
            self.namespaces[METAMODEL_NAMESPACE_NAME] = METAMODEL_NAMESPACE
        self.meta = Namespace(self.namespaces.join(self.namespaces[METAMODEL_NAMESPACE_NAME], ''))  # URI for the metamodel
        self.base = Namespace(self.namespaces.join(self.namespaces._base, ''))    # Base URI for what is being modeled

    def visit_schema(self, **_):
        # Adjust the schema context to include the base model URI
        context = self.shex['@context']
        self.shex['@context'] = [context, {'@base': self.namespaces._base}]
        # Emit all of the type definitions
        for typ in self.schema.types.values():
            model_uri = self._class_or_type_uri(typ)
            if typ.uri:
                typ_type_uri = self.namespaces.uri_for(typ.uri)
                if typ_type_uri in (XSD.anyURI, SHEX.iri):
                    self.shapes.append(NodeConstraint(id=model_uri, nodeKind="iri"))
                elif typ_type_uri == SHEX.nonLiteral:
                    self.shapes.append(NodeConstraint(id=model_uri, nodeKind="nonliteral"))
                else:
                    self.shapes.append(NodeConstraint(id=model_uri, datatype=self.namespaces.uri_for(typ.uri)))
            else:
                typeof_uri = self._class_or_type_uri(typ.typeof)
                self.shapes.append(Shape(id=model_uri, expression=typeof_uri))

    def visit_class(self, cls: ClassDefinition) -> bool:
        self.shape = Shape()

        # Start with all the parent classes, mixins and appytos
        struct_ref_list = [cls.is_a] if cls.is_a else []
        struct_ref_list += cls.mixins
        if cls.name in self.synopsis.applytorefs:
            for applier in self.synopsis.applytorefs[cls.name].classrefs:
                struct_ref_list.append(applier)
        for sr in struct_ref_list:
            self._add_constraint(self._class_or_type_uri(sr, '_tes'))
            self._add_constraint(self._type_arc(self.schema.classes[sr].class_uri, opt=True))
        return True

    def end_class(self, cls: ClassDefinition) -> None:
        # On entry self.shape contains all of the triple expressions that define the body of the shape

        # Finish off the shape definition itself

        # If there is nothing yet, we're at the very root of things.  Add in a final catch-all for any additional
        # type arcs.  NOTE: Here is where you can sink other things as well if you want to ignore categories of things
        if self.shape.expression is None:
            self._add_constraint(TripleConstraint(predicate=RDF.type, min=0, max=-1))
        self.shape.expression.id = self._class_or_type_uri(cls, '_tes')
        self.shape.expression = EachOf(expressions=[self.shape.expression,
                                                    self._type_arc(cls.class_uri,
                                                                   not bool(self.class_identifier(cls)))])
        self.shape.closed = not (cls.abstract or cls.mixin)

        # If this class has subtypes, define the class as the union of its subtypes and itself (if not abstract)
        if cls.name in self.synopsis.isarefs:
            childrenExprs = []
            for child_classname in sorted(list(self.synopsis.isarefs[cls.name].classrefs)):
                childrenExprs.append(self._class_or_type_uri(child_classname))
            if not cls.abstract or len(childrenExprs) == 1:
                childrenExprs.insert(0, self.shape)
                self.shapes.append(ShapeOr(id=self._class_or_type_uri(cls), shapeExprs=childrenExprs))
            else:
                self.shapes.append(ShapeOr(id=self._class_or_type_uri(cls), shapeExprs=childrenExprs))
                self.shape.id = self._class_or_type_uri(cls, "_struct")
                self.shapes.append(self.shape)
        else:
            self.shape.id = self._class_or_type_uri(cls)
            self.shapes.append(self.shape)

    def visit_class_slot(self, cls: ClassDefinition, aliased_slot_name: SlotDefinitionName, slot: SlotDefinition) \
            -> None:
        if not (slot.identifier or slot.abstract):
            constraint = TripleConstraint()
            self._add_constraint(constraint)
            constraint.predicate = self.namespaces.uri_for(slot.slot_uri)
            constraint.min = int(bool(slot.required))
            constraint.max = 1 if not slot.multivalued else -1
            constraint.valueExpr = self._class_or_type_uri(slot.range)

    def end_schema(self, output: Optional[str] = None, **_) -> None:
        self.shex.shapes = self.shapes
        shex = as_json(self.shex)
        if self.format == 'rdf':
            g = Graph()
            g.parse(data=shex, format="json-ld")
            g.bind('owl', OWL)
            shex = g.serialize(format='turtle').decode()
        elif self.format == 'shex':
            g = Graph()
            self.namespaces.load_graph(g)
            shex = str(ShExC(self.shex, base=sfx(self.namespaces._base), namespaces=g))
        if output:
            with open(output, 'w') as outf:
                outf.write(shex)
        else:
            print(shex)

    def _class_or_type_uri(self, item: Union[TypeDefinition, ClassDefinition, ElementName],
                           suffix: Optional[str] = '') -> URIorCURIE:
        if isinstance(item, (TypeDefinition, ClassDefinition)):
            cls_or_type = item
        else:
            cls_or_type = self.class_or_type_for(item)
        return self.namespaces.uri_for(
            self.namespaces.uri_or_curie_for(self.schema_defaults[cls_or_type.from_schema],
                                             camelcase(cls_or_type.name) + suffix))

    def _slot_uri(self, name: str, suffix: Optional[str] = '') -> URIorCURIE:
        slot = self.schema.slots[name]
        return self.namespaces.uri_for(
            self.namespaces.uri_or_curie_for(self.schema_defaults[slot.from_schema], camelcase(name) + suffix))

    def _add_constraint(self, constraint) -> None:
        # No constraints
        if not self.shape.expression:
            self.shape.expression = constraint
        # One constraint
        elif not isinstance(self.shape.expression, EachOf):
            self.shape.expression = EachOf(expressions=[self.shape.expression, constraint])
        # Two or more constraints
        else:
            self.shape.expression.expressions.append(constraint)

    def _type_arc(self, target: URIorCURIE, opt: bool = False) -> TripleConstraint:
        return TripleConstraint(predicate=RDF.type,
                                valueExpr=NodeConstraint(values=[IRIREF(self.namespaces.uri_for(target))]),
                                min=0 if opt else 1)


@shared_arguments(ShExGenerator)
@click.command()
@click.option("-o", "--output", help="Output file name")
def cli(yamlfile, **args):
    """ Generate a ShEx Schema for a  biolink model """
    print(ShExGenerator(yamlfile, **args).serialize(**args))



from dataclasses import dataclass
from typing import Type, List, Optional

from rdflib import URIRef


@dataclass
class Slot:
    """ Runtime slot definition """
    uri: URIRef
    name: str
    curie: Optional[str]
    model_uri: URIRef

    domain: Optional[Type]
    range: Type
    mappings: Optional[List[URIRef]] = None



# Auto generated from .yaml by pythongen.py version:
# Generation date:
# Schema: ifabsent
#
# id: http://example.org/tests/ifabsent
# description:
# license: https://creativecommons.org/publicdomain/zero/1.0/

import dataclasses
import sys
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from biolinkml.utils.slot import Slot
from biolinkml.utils.metamodelcore import empty_list, empty_dict, bnode
from biolinkml.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
if sys.version_info < (3, 7, 6):
    from biolinkml.utils.dataclass_extensions_375 import dataclasses_init_fn_with_kwargs
else:
    from biolinkml.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from biolinkml.utils.formatutils import camelcase, underscore, sfx
from rdflib import Namespace, URIRef
from biolinkml.utils.curienamespace import CurieNamespace
from includes.types import String

metamodel_version = "1.4.3"

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
BIOLINKML = CurieNamespace('biolinkml', 'https://w3id.org/biolink/biolinkml/')
SKOS = CurieNamespace('skos', 'http://www.w3.org/2004/02/skos/core#')
TEST = CurieNamespace('test', 'http://example.org/tests/ifabsent/')
DEFAULT_ = TEST


# Types

# Class references



@dataclass
class C1(YAMLRoot):
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = TEST.C1
    class_class_curie: ClassVar[str] = "test:C1"
    class_name: ClassVar[str] = "c1"
    class_model_uri: ClassVar[URIRef] = TEST.C1

    s1: Optional[str] = SKOS.label
    s2: Optional[str] = SKOS.definition


# Slots
class slots:
    pass

slots.s1 = Slot(uri=TEST.s1, name="s1", curie=TEST.curie('s1'),
                      model_uri=TEST.s1, domain=None, range=Optional[str])

slots.s2 = Slot(uri=TEST.s2, name="s2", curie=TEST.curie('s2'),
                      model_uri=TEST.s2, domain=None, range=Optional[str])



# id: http://example.org/tests/types
# description:
# license:

import dataclasses
import sys
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from biolinkml.utils.slot import Slot
from biolinkml.utils.metamodelcore import empty_list, empty_dict, bnode
from biolinkml.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
if sys.version_info < (3, 7, 6):
    from biolinkml.utils.dataclass_extensions_375 import dataclasses_init_fn_with_kwargs
else:
    from biolinkml.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from biolinkml.utils.formatutils import camelcase, underscore, sfx
from rdflib import Namespace, URIRef
from biolinkml.utils.curienamespace import CurieNamespace
from biolinkml.utils.metamodelcore import Bool

metamodel_version = "1.4.3"

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
METATYPE = CurieNamespace('metatype', 'https://w3id.org/biolink/biolinkml/types/')
XSD = CurieNamespace('xsd', 'http://www.w3.org/2001/XMLSchema#')
DEFAULT_ = CurieNamespace('', 'http://example.org/tests/types/')


# Types
class String(str):
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "string"
    type_model_uri = URIRef("http://example.org/tests/types/String")


class Boolean(Bool):
    type_class_uri = XSD.boolean
    type_class_curie = "xsd:boolean"
    type_name = "boolean"
    type_model_uri = URIRef("http://example.org/tests/types/Boolean")


class BooleanType(Boolean):
    type_class_uri = XSD.boolean
    type_class_curie = "xsd:boolean"
    type_name = "boolean type"
    type_model_uri = URIRef("http://example.org/tests/types/BooleanType")


class StringType(String):
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "string type"
    type_model_uri = URIRef("http://example.org/tests/types/StringType")


# Class references





# Slots
class slots:
    pass





# id: http://example.org/example/multi_id
# description:
# license:

import dataclasses
import sys
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from biolinkml.utils.slot import Slot
from biolinkml.utils.metamodelcore import empty_list, empty_dict, bnode
from biolinkml.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
if sys.version_info < (3, 7, 6):
    from biolinkml.utils.dataclass_extensions_375 import dataclasses_init_fn_with_kwargs
else:
    from biolinkml.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from biolinkml.utils.formatutils import camelcase, underscore, sfx
from rdflib import Namespace, URIRef
from biolinkml.utils.curienamespace import CurieNamespace
from biolinkml.utils.metamodelcore import URIorCURIE

metamodel_version = "1.4.3"

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
XSD = CurieNamespace('xsd', 'http://www.w3.org/2001/XMLSchema#')
DEFAULT_ = CurieNamespace('', 'http://example.org/example/multi_id/')


# Types
class Uri(URIorCURIE):
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uri"
    type_model_uri = URIRef("http://example.org/example/multi_id/Uri")


class IdentifierType(Uri):
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "identifier type"
    type_model_uri = URIRef("http://example.org/example/multi_id/IdentifierType")


# Class references
class NamedThingId(URIorCURIE):
    pass


class SequenceVariantId(NamedThingId):
    pass


@dataclass
class NamedThing(YAMLRoot):
    _inherited_slots: ClassVar[List[str]] = ["node_property", "id"]

    class_class_uri: ClassVar[URIRef] = URIRef("http://example.org/example/multi_id/NamedThing")
    class_class_curie: ClassVar[str] = None
    class_name: ClassVar[str] = "named thing"
    class_model_uri: ClassVar[URIRef] = URIRef("http://example.org/example/multi_id/NamedThing")

    id: Union[URIorCURIE, NamedThingId]
    node_property: Optional[Union[URIorCURIE, IdentifierType]] = None
    not_overridden: Optional[Union[URIorCURIE, IdentifierType]] = None

    def __post_init__(self, **kwargs: Dict[str, Any]):
        if self.id is None:
            raise ValueError(f"id must be supplied")
        if not isinstance(self.id, NamedThingId):
            self.id = NamedThingId(self.id)
        if self.node_property is not None and not isinstance(self.node_property, IdentifierType):
            self.node_property = IdentifierType(self.node_property)
        if self.not_overridden is not None and not isinstance(self.not_overridden, IdentifierType):
            self.not_overridden = IdentifierType(self.not_overridden)
        super().__post_init__(**kwargs)


@dataclass
class SequenceVariant(NamedThing):
    _inherited_slots: ClassVar[List[str]] = ["node_property", "id"]

    class_class_uri: ClassVar[URIRef] = URIRef("http://example.org/example/multi_id/SequenceVariant")
    class_class_curie: ClassVar[str] = None
    class_name: ClassVar[str] = "sequence variant"
    class_model_uri: ClassVar[URIRef] = URIRef("http://example.org/example/multi_id/SequenceVariant")

    id: Union[URIorCURIE, SequenceVariantId] = None
    node_property: Optional[Union[URIorCURIE, IdentifierType]] = None

    def __post_init__(self, **kwargs: Dict[str, Any]):
        if self.id is None:
            raise ValueError(f"id must be supplied")
        if not isinstance(self.id, SequenceVariantId):
            self.id = SequenceVariantId(self.id)
        if self.node_property is not None and not isinstance(self.node_property, IdentifierType):
            self.node_property = IdentifierType(self.node_property)
        super().__post_init__(**kwargs)



# Slots
class slots:
    pass

slots.node_property = Slot(uri=DEFAULT_.node_property, name="node property", curie=DEFAULT_.curie('node_property'),
                      model_uri=DEFAULT_.node_property, domain=NamedThing, range=Optional[Union[URIorCURIE, IdentifierType]])

slots.not_overridden = Slot(uri=DEFAULT_.not_overridden, name="not overridden", curie=DEFAULT_.curie('not_overridden'),
                      model_uri=DEFAULT_.not_overridden, domain=NamedThing, range=Optional[Union[URIorCURIE, IdentifierType]])

slots.id = Slot(uri=DEFAULT_.id, name="id", curie=DEFAULT_.curie('id'),
                      model_uri=DEFAULT_.id, domain=NamedThing, range=Union[URIorCURIE, NamedThingId])

slots.sequence_variant_id = Slot(uri=DEFAULT_.id, name="sequence variant_id", curie=DEFAULT_.curie('id'),
                      model_uri=DEFAULT_.sequence_variant_id, domain=SequenceVariant, range=Union[URIorCURIE, SequenceVariantId])

slots.sequence_variant_node_property = Slot(uri=DEFAULT_.node_property, name="sequence variant_node property", curie=DEFAULT_.curie('node_property'),
                      model_uri=DEFAULT_.sequence_variant_node_property, domain=SequenceVariant, range=Optional[Union[URIorCURIE, IdentifierType]])


import os
import unittest

from biolinkml.utils.schemaloader import SchemaLoader
from jsonasobj import as_dict, as_json, JsonObj, load
from tests.test_utils import inputdir, outputdir


class ResolverTestCase(unittest.TestCase):

    def test_default_range(self):
        """ Validate default slot range settings """
        schema = SchemaLoader(os.path.join(inputdir, 'resolver1.yaml')).resolve()
        self.assertEqual({'s1':'t1', 's2':'t2'}, {slot.name: slot.range for slot in schema.slots.values()})
        schema = SchemaLoader(os.path.join(inputdir, 'resolver2.yaml')).resolve()
        self.assertEqual({'s1': 'string', 's2': 't2'}, {slot.name: slot.range for slot in schema.slots.values()})

    def test_type_uri(self):
        """ Validate type URI's and the fact that they aren't inherited """
        schema = SchemaLoader(os.path.join(inputdir, 'resolver2.yaml')).resolve()
        self.assertEqual({'string': 'xsd:string', 't1': 'xsd:string', 't2': 'xsd:int', 't3': 'xsd:string'},
                         {t.name: t.uri for t in schema.types.values()})

    def test_element_slots(self):
        """ Test all element slots and their inheritence """
        schema = SchemaLoader(os.path.join(inputdir, 'resolver3.yaml')).resolve()
        x = {k:v for k, v in as_dict(schema.slots['s1']).items() if v is not None and v != []}
        outfile = os.path.join(outputdir, 'resolver3.json')
        if not os.path.exists(outfile):
            with open(outfile, 'w') as f:
                f.write(as_json(JsonObj(**x)))
                self.fail(f"File {outfile} created - rerun test")
        with open(outfile) as f:
            expected = as_dict(load(f))

        self.assertEqual(expected, x)


if __name__ == '__main__':
    unittest.main()



import unittest

from biolinkml import INCLUDES_DIR, META_BASE_URI, MODULE_DIR
from biolinkml.generators.jsonldcontextgen import ContextGenerator
from tests import targetdir
from tests.utils.generator_utils import GeneratorTestCase, BIOLINK_IMPORT_MAP
from tests.utils.metadata_filters import ldcontext_metadata_filter


class ContextTestCase(GeneratorTestCase):
    """ Generate the context.jsonld for all of the models and compare them against what has been published """
    target_path = targetdir

    def test_types_context(self):
        self.source_path = INCLUDES_DIR
        self.model_path = INCLUDES_DIR
        self.model_name = 'types'
        self.single_file_generator('context.jsonld', ContextGenerator, filtr=ldcontext_metadata_filter)

    def test_mappings_context(self):
        self.source_path = INCLUDES_DIR
        self.model_path = INCLUDES_DIR
        self.model_name = 'mappings'
        self.importmap = BIOLINK_IMPORT_MAP
        self.single_file_generator('context.jsonld', ContextGenerator, filtr=ldcontext_metadata_filter)

    def test_metamodel_context(self):
        self.source_path = MODULE_DIR
        self.model_path = MODULE_DIR
        self.model_name = 'meta'
        self.output_name = ''
        self.importmap = BIOLINK_IMPORT_MAP
        self.single_file_generator('context.jsonld', ContextGenerator, serialize_args=dict(base=META_BASE_URI),
                                   filtr=ldcontext_metadata_filter)


if __name__ == '__main__':
    unittest.main()
